@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 #
-# Copyright (c) 2016-2024, Babak Farrokhi
+# Copyright (c) 2016-2025, Babak Farrokhi
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -33,15 +33,15 @@ import signal
 import socket
 import sys
 import time
-import httpx
 from statistics import stdev
 
 import dns.flags
 import dns.resolver
+import httpx
 
-from util.dns import PROTO_UDP, PROTO_TCP, PROTO_TLS, PROTO_HTTPS, PROTO_QUIC, proto_to_text, unsupported_feature, \
-    random_string, getDefaultPort, valid_rdatatype
-from util.shared import __version__
+from dnsdiag.dns import PROTO_UDP, PROTO_TCP, PROTO_TLS, PROTO_HTTPS, PROTO_QUIC, PROTO_HTTP3, proto_to_text, \
+    unsupported_feature, random_string, getDefaultPort, valid_rdatatype
+from dnsdiag.shared import __version__
 
 __author__ = 'Babak Farrokhi (babak@farrokhi.net)'
 __license__ = 'BSD'
@@ -62,6 +62,7 @@ Usage: %s [-46aDeEFhLmqnrvTQxXH] [-i interval] [-w wait] [-p dst_port] [-P src_p
   -T, --tcp         Use TCP as the transport protocol
   -X, --tls         Use TLS as the transport protocol
   -H, --doh         Use HTTPS as the transport protocol (DoH)
+  -3, --http3       Use HTTP/3 as the transport protocol (DoH3)
   -Q, --doq         Use QUIC as the transport protocol (DoQ)
   -4, --ipv4        Use IPv4 as the network protocol
   -6, --ipv6        Use IPv6 as the network protocol
@@ -154,11 +155,11 @@ def main():
     qname = 'wikipedia.org'
 
     try:
-        opts, args = getopt.getopt(sys.argv[1:], "qhc:s:t:w:i:vp:P:S:TQ46meDFXHrnEC:Lxa",
+        opts, args = getopt.getopt(sys.argv[1:], "qhc:s:t:w:i:vp:P:S:TQ346meDFXHrnEC:Lxa",
                                    ["help", "count=", "server=", "quiet", "type=", "wait=", "interval=", "verbose",
                                     "port=", "srcip=", "tcp", "ipv4", "ipv6", "cache-miss", "srcport=", "edns",
                                     "dnssec", "flags", "norecurse", "tls", "doh", "nsid", "ede", "class=", "ttl",
-                                    "expert", "answer", "quic"])
+                                    "expert", "answer", "quic", "http3"])
     except getopt.GetoptError as err:
         # print help information and exit:
         print_stderr(err, False)  # will print something like "option -a not recognized"
@@ -238,6 +239,11 @@ def main():
             if use_default_dst_port:
                 dst_port = getDefaultPort(proto)
 
+        elif o in ("-3", "--http3"):
+            proto = PROTO_HTTP3
+            if use_default_dst_port:
+                dst_port = getDefaultPort(proto)
+
         elif o in ("-4", "--ipv4"):
             af = socket.AF_INET
 
@@ -346,6 +352,18 @@ def main():
                 else:
                     unsupported_feature("DNS-over-HTTPS (DoH)")
 
+            elif proto is PROTO_HTTP3:
+                if hasattr(dns.query, 'quic'):
+                    try:
+                        answers = dns.query.https(query, dnsserver, timeout=timeout, port=dst_port,
+                                                  source=src_ip, source_port=src_port,
+                                                  http_version=dns.query.HTTPVersion.H3)
+                    except ConnectionRefusedError:
+                        print_stderr(f"The server did not respond to DNS-over-HTTPS/3 on port {dst_port}",
+                                     should_die=True)
+                else:
+                    unsupported_feature("DNS-over-HTTPS/3 (DoH3)")
+
             elif proto is PROTO_QUIC:
                 if hasattr(dns.query, 'quic'):
                     try:
@@ -353,6 +371,9 @@ def main():
                                                  source=src_ip, source_port=src_port)
                     except dns.exception.Timeout:
                         print_stderr(f"The server did not respond to DoQ on port {dst_port}", should_die=True)
+                    except ConnectionRefusedError:
+                        print_stderr(f"The server did not respond to DNS-over-HTTPS/3 on port {dst_port}",
+                                     should_die=True)
                 else:
                     unsupported_feature("DNS-over-QUIC (DoQ)")
 
@@ -401,6 +422,8 @@ def main():
                 if show_flags:
                     ans_flags = dns.flags.to_text(answers.flags)
                     edns_flags = dns.flags.edns_to_text(answers.ednsflags)
+                    if want_dnssec and not (answers.flags & dns.flags.AD):
+                        ans_flags += " --"  # add padding to printer output when dnssec is requested, but AD flag is not set
                     extras += " [%s]" % " ".join([ans_flags, edns_flags]).rstrip(' ')  # show both regular + edns flags
 
                 if want_nsid:
@@ -412,12 +435,11 @@ def main():
                 if show_ede:
                     for ans_opt in answers.options:  # EDE response is optional, but print if there is one
                         if ans_opt.otype == dns.edns.EDE:
-                            extras += " [EDE %d: %s]" % (ans_opt.code, ans_opt.text)
+                            extras += " [EDE %d: \"%s\"]" % (ans_opt.code, ans_opt.text or "")
 
                 if show_answer:  # The answer should be displayed at the rightmost
                     for ans in answers.answer:
                         if ans.rdtype == dns.rdatatype.from_text(rdatatype):  # is this the answer to our question?
-                            # extras += " [%s]" % ans[0]
                             extras += " [RDATA: %s]" % ans[0]
                             break
 
